/*
 * XPathProcessor - ESIAL PROJET SD 2011
 * Copyright (C) 2011 Julien VAUBOURG / Tristan STEF
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package Processeur;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;

public class Analyseur {
	public static UniteLexicale analyser(String fichierEntree, String nomAxiome) {
		BufferedReader lecture = null;
		UniteLexicale axiome, rAxiome = null;
		int c, ligne = 1;
		
		try {
			// Lecture du fichier XML a verifier et mettre en memoire
			lecture = EntreesSorties.lire(fichierEntree);

			// Racine du document imposee (du package Grammaire)
			// EXCEPTION : Verifie si l'axiome existe bien dans la grammaire
			axiome = (UniteLexicale) Class.forName("Grammaire." + EntreesSorties.ajouterMajuscule(nomAxiome)).newInstance();

			// Analyseurs
			AnalyseurLexical analyseurLexical = new AnalyseurLexical();
			AnalyseurSyntaxique analyseurSyntaxique = new AnalyseurSyntaxique(axiome);

			// Lecture du fichier caractere par caractere  
			while ((c = lecture.read()) > -1) {
				
				// EXCEPTION : Verifie la syntaxe XML (exception lexicale)
				// EXCEPTION : Une balise inconnue de la grammaire a ete trouvee (exception lexicale)
				// EXCEPTION : Un attribut a ete trouve sur une balise qui ne l'accepte pas (exception lexicale)
				// EXCEPTION : Un attribut conforme a ete trouve plus d'une fois sur une meme balise (exception lexicale)
				String terminal = analyseurLexical.interpreter((char) c);

				// L'analyseur lexical a detecte une balise ouvrante conforme et connue
				if(analyseurLexical.getTypeTerminal() == AnalyseurLexical.TypeTerminal.B_OUVRANTE) {

					// EXCEPTION : Verifie l'ordre des balises (exception syntaxique)
					// EXCEPTION : Le fichier tente de declarer une balise <text> ou <void> explicite
					analyseurSyntaxique.insererTerminal(terminal, analyseurLexical.getAttributs());

				// L'analyseur lexical a detecte du texte brut
				} else if(analyseurLexical.getTypeTerminal() == AnalyseurLexical.TypeTerminal.TEXTE) {
					
					// EXCEPTION : Verifie si un noeud peut bien contenir du texte (exception syntaxique) 
					analyseurSyntaxique.insererTerminalTexte(terminal);

				// L'analyseur lexical a detecte une balise fermante conforme et connue
				} else if(analyseurLexical.getTypeTerminal() == AnalyseurLexical.TypeTerminal.B_FERMANTE) {
					
					// EXCEPTION : Verifie si une balise contient bien le minimum de ce qu'elle doit contenir (exception syntaxique)
					analyseurSyntaxique.fermer(terminal);
				}
				
				// Numeros de lignes pour le debogage (TODO: a adapter pour les OS alternatifs)
				if(c == '\n')
					ligne++;
				
				// On fixe le retour
				rAxiome = axiome;
			}

		} catch (InstantiationException e) {
			System.err.println("(" + fichierEntree + ":" + ligne + ") " + e.getMessage());
			
		} catch (ExceptionLexicale e) {
			System.err.println("(" + fichierEntree + ":" + ligne + ") " + e.getMessage());

		} catch (ExceptionSyntaxique e) {
			System.err.println("(" + fichierEntree + ":" + ligne + ") " + e.getMessage());

		} catch (FileNotFoundException e) {
			System.err.println("Le fichier " + fichierEntree + " est introuvable.");

		} catch (IOException e) {
			System.err.println("Problème à la lecture du fichier " + fichierEntree + ".");

		} catch (IllegalAccessException e) {
			System.err.println("L'axiome " + nomAxiome + " indiqué ne correspond à aucune classe valide du package de la grammaire.");

		} catch (ClassNotFoundException e) {
			System.err.println("L'axiome " + nomAxiome + " indiqué ne correspond à aucune classe du package de la grammaire.");

		} finally {
			try {
				if(lecture != null)
					lecture.close();
			} catch (IOException e) {
				System.err.println("Problème à la fermeture du fichier " + fichierEntree + ".");
			}
		}
		
		return rAxiome;
	}
}
