/*
 * XPathProcessor - ESIAL PROJET SD 2011
 * Copyright (C) 2011 Julien VAUBOURG / Tristan STEF
 * Contact <julien@vaubourg.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package Processeur;

import java.io.*;
import java.util.*;

public class EntreesSorties {
	
	/**
	 * Ouvre un flux sur un fichier.
	 * @param fichier Nom du fichier
	 * @return Flux du fichier
	 * @throws FileNotFoundException Fichier non trouve
	 */
	public static BufferedReader lire(String fichier) throws FileNotFoundException {
		InputStream is = new FileInputStream(fichier);
		InputStreamReader isr = new InputStreamReader(is);

		return new BufferedReader(isr);
	}
	
	/**
	 * Met la premiere lettre d'une chaine en majuscule.
	 * @param mot Chaine
	 * @return Chaine avec une majuscule
	 */
	public static String ajouterMajuscule(String mot) {
		return Character.toUpperCase(mot.charAt(0)) + mot.substring(1);
	}
	
	/**
	 * Demande a l'utilisateur de saisir une requete XPath
	 * @return Requete saisie
	 */
	public static String saisirRequeteXPath() {
		System.out.print("(XPath) >> ");
		return new Scanner(System.in).nextLine();
	}
	
	/**
	 * Ecrit dans un fichier les sorties XML des noeuds passes
	 * @param fichier Nom du fichier dans lequel ecrire
	 * @param noeuds Noeuds a ecrire
	 * @throws IOException Impossible d'ecrire dans le fichier
	 */
	public static void ecrire(String fichier, ArrayList<UniteLexicale> noeuds) throws IOException {
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(fichier)));

		for(UniteLexicale noeud : noeuds)
			pw.write(noeud.xml());

		pw.close();
	}
}