package Tests;

import Processeur.AnalyseurLexical;
import Processeur.ExceptionLexicale;
import junit.framework.TestCase;

public class AnalyseurLexicalTest extends TestCase {
	protected AnalyseurLexical al;
	protected String strTest;

	@Override
	protected void setUp() {
		al = new AnalyseurLexical();
		strTest = "";
	}

	protected String interpreterStrTest() throws ExceptionLexicale {
		int i;
		String terminal = null;
		
		for(i = 0; terminal == null && i < strTest.length(); i++)
			terminal = al.interpreter(strTest.charAt(i));

		strTest = strTest.substring(i);

		return terminal;
	}
	
	public void testInterpreterBaliseOuvranteBienFormee() {
		String terminal= null;

		// Balise seule
		
		strTest = "<personne>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertTrue(strTest.isEmpty());
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		
		// Balise suivie de texte
		
		strTest = "<personne>du texte";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("du texte", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		
		// Balise suivie d'une autre balise
		
		strTest = "<personne><naissance>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("<naissance>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		
		// Balise suivie d'une balise malformee
		
		strTest = "<personne><impossible !>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("<impossible !>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		
		// Balise suivie d'une balise ouvrante inconnue de la grammaire
		
		strTest = "<personne><inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("<inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		
		// Balise suivie d'une balise fermante inconnue de la grammaire
		
		strTest = "<personne></inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("</inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
	}
	
	public void testInterpreterBaliseOuvranteBienFormeeAvecAttribut() {
		String terminal= null;
		
		// Balise seule
		
		strTest = "<telephone type=\"gsm\">";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("telephone", terminal);
		assertTrue(strTest.isEmpty());
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		assertEquals(al.getAttributs().size(), 1);
		assertEquals(al.getAttributs().get("type"), "gsm");
		
		// Balise seule, attribut inconnu de la balise
		
		strTest = "<telephone style=\"gsm\">";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("comme un attribut valide"));
		}
		
		// Balise seule avec un attribut en double
		
		strTest = "<telephone type=\"gsm\" type=\"fix\">";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("plus d'un attribut"));
		}

		// Balise suivie de texte
		
		strTest = "<telephone type=\"gsm\">du texte";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("telephone", terminal);
		assertEquals("du texte", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		assertEquals(al.getAttributs().size(), 1);
		assertEquals(al.getAttributs().get("type"), "gsm");
		
		// Balise suivie d'une autre balise
		
		strTest = "<telephone type=\"gsm\"><naissance>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("telephone", terminal);
		assertEquals("<naissance>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		assertEquals(al.getAttributs().size(), 1);
		assertEquals(al.getAttributs().get("type"), "gsm");
		
		// Balise suivie d'une balise malformee
		
		strTest = "<telephone type=\"gsm\"><impossible !>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("telephone", terminal);
		assertEquals("<impossible !>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		assertEquals(al.getAttributs().size(), 1);
		assertEquals(al.getAttributs().get("type"), "gsm");
		
		// Balise suivie d'une balise ouvrante inconnue de la grammaire
		
		strTest = "<telephone type=\"gsm\"><inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("telephone", terminal);
		assertEquals("<inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		assertEquals(al.getAttributs().size(), 1);
		assertEquals(al.getAttributs().get("type"), "gsm");
		
		// Balise suivie d'une balise fermante inconnue de la grammaire
		
		strTest = "<telephone type=\"gsm\"></inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("telephone", terminal);
		assertEquals("</inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		assertEquals(al.getAttributs().size(), 1);
		assertEquals(al.getAttributs().get("type"), "gsm");
	}
	
	public void testInterpreterBaliseFermanteBienFormee() {
		String terminal= null;

		// Balise seule
		
		strTest = "</personne>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertTrue(strTest.isEmpty());
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());
		
		// Balise suivie de texte
		
		strTest = "</personne>du texte";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("du texte", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());
		
		// Balise suivie d'une autre balise
		
		strTest = "</personne><naissance>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("<naissance>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());
		
		// Balise suivie d'une balise malformee
		
		strTest = "</personne><impossible !>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("<impossible !>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());
		
		// Balise suivie d'une balise ouvrante inconnue de la grammaire
		
		strTest = "</personne><inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("<inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());

		// Balise suivie d'une balise fermante inconnue de la grammaire
		
		strTest = "</personne></inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("personne", terminal);
		assertEquals("</inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());
	}
	
	public void testInterpreterTexte() {
		String terminal= null;
				
		// Texte suivi d'une balise
		
		strTest = "du texte<naissance>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("du texte", terminal);
		assertEquals("naissance>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.TEXTE, al.getTypeTerminal());
		
		setUp();
		
		// Texte suivi d'une balise malformee
		
		strTest = "du texte<impossible !>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("du texte", terminal);
		assertEquals("impossible !>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.TEXTE, al.getTypeTerminal());
		
		setUp();
		
		// Texte suivi d'une balise ouvrante inconnue de la grammaire
		
		strTest = "du texte<inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("du texte", terminal);
		assertEquals("inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.TEXTE, al.getTypeTerminal());
		
		setUp();

		// Texte suivi d'une balise fermante inconnue de la grammaire
		
		strTest = "du texte</inconnue>";
		
		try {
			terminal = interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals("du texte", terminal);
		assertEquals("/inconnue>", strTest);
		assertEquals(AnalyseurLexical.TypeTerminal.TEXTE, al.getTypeTerminal());
	}
	
	public void testInterpreterBaliseMalformee() {

		// Balise malformee seule
		
		strTest = "<impossible !>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("cohérente"));
		}
		
		// Balise malformee suivie de texte
		
		strTest = "<impossible !>du texte";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("cohérente"));
		}
		
		// Balise malformee suivie d'une balise bien formee
		
		strTest = "<impossible !><naissance>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("cohérente"));
		}
		
		// Balise malformee suivie d'une autre balise malformee
		
		strTest = "<impossible !><impossible !>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("cohérente"));
		}
		
		// Balise malformee suivie d'une balise ouvrante inconnue de la grammaire
		
		strTest = "<impossible !><inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("cohérente"));
		}
		
		// Balise malformee suivie d'une balise fermante inconnue de la grammaire
		
		strTest = "<impossible !></inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("cohérente"));
		}
	}
	
	public void testInterpreterBaliseOuvranteInconnue() {
		
		// Balise ouvrante inconnue seule
		
		strTest = "<inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise ouvrante inconnue suivie de texte
		
		strTest = "<inconnue>du texte";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise ouvrante inconnue suivie d'une balise
		
		strTest = "<inconnue><naissance>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise ouvrante inconnue suivie d'une balise malformee
		
		strTest = "<inconnue><impossible !>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise ouvrante inconnue suivie d'une autre balise ouvrante inconnue de la grammaire
		
		strTest = "<inconnue><inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise ouvrante inconnue suivie d'une balise fermante inconnue de la grammaire
		
		strTest = "<inconnue></inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
	}
	
	public void testInterpreterBaliseFermanteInconnue() {
		
		// Balise fermante inconnue seule
		
		strTest = "</inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise fermante inconnue suivie de texte
		
		strTest = "</inconnue>du texte";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise fermante inconnue suivie d'une balise
		
		strTest = "</inconnue><naissance>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise fermante inconnue suivie d'une balise malformee
		
		strTest = "</inconnue><impossible !>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise fermante inconnue suivie d'une balise ouvrante inconnue de la grammaire
		
		strTest = "</inconnue><inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
		
		// Balise fermante inconnue suivie d'une autre balise fermante inconnue de la grammaire
		
		strTest = "</inconnue></inconnue>";
		
		try {
			interpreterStrTest();
			fail();
		} catch (ExceptionLexicale e) {
			assertTrue(e.getMessage().contains("inconnu"));
		}
	}

	public void testGetTypeTerminal() {
		
		// Type balise ouvrante

		strTest = "<personne>";
		
		try {
			interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}
		
		assertEquals(AnalyseurLexical.TypeTerminal.B_OUVRANTE, al.getTypeTerminal());
		
		// Type balise fermante

		strTest = "</personne>";
		
		try {
			interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}
		
		assertEquals(AnalyseurLexical.TypeTerminal.B_FERMANTE, al.getTypeTerminal());
		
		// Type texte

		strTest = "du texte<";

		try {
			interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}
		
		assertEquals(AnalyseurLexical.TypeTerminal.TEXTE, al.getTypeTerminal());
		
		setUp();
		
		// Type intermediaire

		strTest = "<person";
		
		try {
			interpreterStrTest();
		} catch (ExceptionLexicale e) {
			fail(e.getMessage());
		}

		assertEquals(AnalyseurLexical.TypeTerminal.NT, al.getTypeTerminal());
	}
}