package Tests;

import java.util.HashMap;

import Grammaire.Annuaire;
import Processeur.AnalyseurSyntaxique;
import Processeur.ExceptionSyntaxique;
import Processeur.UniteLexicale;
import junit.framework.TestCase;

public class AnalyseurSyntaxiqueTest extends TestCase {
	protected AnalyseurSyntaxique as;
	protected UniteLexicale axiome;
	protected HashMap<String, String> aucunAttribut, unAttribut;

	@Override
	protected void setUp() {
		axiome = new Annuaire();
		as = new AnalyseurSyntaxique(axiome);
		
		aucunAttribut = new HashMap<String, String>();
		unAttribut = new HashMap<String, String>();
		
		unAttribut.put("type", "gsm");
	}
	
	protected void setUpRacine() {
		setUp();

		try {
			as.insererTerminal("annuaire", aucunAttribut);
		} catch (ExceptionSyntaxique e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
	}

	protected void setUpRacinePersonneNonFermee() {
		setUpRacine();
		setUpPersonneNonFermee();
	}
	
	protected void setUpPersonneNonFermee() {
		setUpPersonneNomPrenom();

		try {
			as.insererTerminal("adresse", aucunAttribut);
			as.insererTerminalTexte("_ADRESSE_");
			as.fermer("adresse");
			
			as.insererTerminal("email", aucunAttribut);
			as.insererTerminalTexte("_EMAIL_");
			as.fermer("email");
			
		} catch (ExceptionSyntaxique e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
	}
	
	protected void setUpRacinePersonne() {
		setUpRacine();
		setUpPersonneNonFermee();
		
		try {
			as.fermer("personne");
		} catch (ExceptionSyntaxique e) {
			e.printStackTrace();
		}
	}

	protected void setUpRacinePersonneVide() {
		setUpRacine();

		try {
			as.insererTerminal("personne", aucunAttribut);

		} catch (ExceptionSyntaxique e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
	}

	protected void setUpRacinePersonneNomPrenom() {
		setUpRacine();
		setUpPersonneNomPrenom();
	}
	
	protected void setUpPersonneNomPrenom() {

		try {
			as.insererTerminal("personne", aucunAttribut);
			
			as.insererTerminal("nom", aucunAttribut);
			as.insererTerminalTexte("_NOM_");
			as.fermer("nom");
			
			as.insererTerminal("prenom", aucunAttribut);
			as.insererTerminalTexte("_PRENOM_");
			as.fermer("prenom");

		} catch (ExceptionSyntaxique e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
	}
	
	public void testInsererTerminalTexte() {
		setUpRacinePersonneVide();
		
		try {
			as.insererTerminal("nom", aucunAttribut);
			as.insererTerminalTexte("_NOM_");
			as.fermer("nom");
			
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
	}

	public void testInsererTerminalAttribut() {
		setUpRacinePersonneNomPrenom();
		
		try {
			as.insererTerminal("adresse", aucunAttribut);
			as.insererTerminalTexte("_ADRESSE_");
			as.fermer("adresse");
			
			as.insererTerminal("telephone", unAttribut);
			as.insererTerminalTexte("_TELEPHONE_");
			as.fermer("telephone");
			
			as.insererTerminal("email", aucunAttribut);
			as.insererTerminalTexte("_EMAIL_");
			as.fermer("email");
			
			as.fermer("personne");
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		assertEquals("<annuaire>\n\t<personne>\n\t\t<nom>_NOM_</nom>\n\t\t<prenom>_PRENOM_</prenom>\n\t\t<adresse>_ADRESSE_</adresse>\n\t\t<telephone type=\"gsm\">_TELEPHONE_</telephone>\n\t\t<email>_EMAIL_</email>\n\t</personne>\n</annuaire>\n", axiome.xml());		
	}
	
	public void testInsererTerminalArbreVide() {
		
		// Ajout d'une premiere balise qui ne correspond pas a la racine definie
		
		try {
			as.insererTerminal("personne", aucunAttribut);
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("racine"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		setUp();
		
		// Ajout d'une premiere balise qui correspond a la racine definie
		
		try {
			as.insererTerminal("annuaire", aucunAttribut);
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
	}
	
	public void testInsererTerminalRacineSeule() {
		
		// Ajout d'une premiere balise fille incompatible
		
		setUpRacine();
		
		try {
			as.insererTerminal("naissance", aucunAttribut);
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("ne peut accepter"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Ajout d'une premiere balise fille compatible
		
		setUpRacine();
				
		try {
			as.insererTerminal("personne", aucunAttribut);
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}

		// Ajout d'une seconde balise fille, incompatible (suivante impossible de la premiere)
		
		setUpRacinePersonne();
		
		try {
			as.insererTerminal("naissance", aucunAttribut);
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("ne peut accepter"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Ajout d'une seconde balise fille, compatible (suivante possible de la premiere)
		
		setUpRacinePersonne();
		
		try {
			as.insererTerminal("personne", aucunAttribut);
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
	}
	
	public void testInsererTerminalAuDelaRacine() {
		
		// Ajout d'une premiere balise fille incompatible
		
		setUpRacinePersonneVide();
		
		try {
			as.insererTerminal("naissance", aucunAttribut);
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("ne peut accepter"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Ajout d'une premiere balise fille compatible
		
		setUpRacinePersonneVide();
				
		try {
			as.insererTerminal("nom", aucunAttribut);
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}

		// Ajout d'une seconde balise fille, incompatible (suivante impossible de la premiere)
		
		setUpRacinePersonneNomPrenom();
		
		try {
			as.insererTerminal("email", aucunAttribut);
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("ne peut accepter"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Ajout d'une seconde balise fille, compatible (suivante possible de la premiere)
		
		setUpRacinePersonneNomPrenom();
		
		try {
			as.insererTerminal("adresse", aucunAttribut);
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
	}

	public void testFermerRacineSeule() {
		
		// Fermeture de la racine seule
		
		setUpRacine();
		
		try {
			as.fermer("annuaire");
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		}
		
		assertEquals("<annuaire>\n</annuaire>\n", axiome.xml());
		
		// Fermeture d'un premier incomplet
		
		setUpRacinePersonneNomPrenom();
		
		try {
			as.fermer("personne");
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("manquants"));
		}
		
		// Fermeture d'un premier complet
		
		setUpRacinePersonneNonFermee();
		
		try {
			as.fermer("personne");
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		}
		
		assertEquals("<annuaire>\n\t<personne>\n\t\t<nom>_NOM_</nom>\n\t\t<prenom>_PRENOM_</prenom>\n\t\t<adresse>_ADRESSE_</adresse>\n\t\t<email>_EMAIL_</email>\n\t</personne>\n</annuaire>\n", axiome.xml());
		
		// Fermeture d'un suivant incomplet
		
		setUpRacinePersonne();
		setUpPersonneNomPrenom();
		
		try {
			as.fermer("personne");
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("manquants"));
		}
		
		// Fermeture d'un suivant complet
		
		setUpRacinePersonne();
		setUpPersonneNonFermee();
		
		try {
			as.fermer("personne");
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		}
		
		assertEquals("<annuaire>\n\t<personne>\n\t\t<nom>_NOM_</nom>\n\t\t<prenom>_PRENOM_</prenom>\n\t\t<adresse>_ADRESSE_</adresse>\n\t\t<email>_EMAIL_</email>\n\t</personne>\n\t<personne>\n\t\t<nom>_NOM_</nom>\n\t\t<prenom>_PRENOM_</prenom>\n\t\t<adresse>_ADRESSE_</adresse>\n\t\t<email>_EMAIL_</email>\n\t</personne>\n</annuaire>\n", axiome.xml());
	}
	
	public void testFermerAuDelaRacine() {
		
		// Fermeture de la racine
		
		setUpRacinePersonne();
		
		try {
			as.fermer("annuaire");
			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		}
		
		assertEquals("<annuaire>\n\t<personne>\n\t\t<nom>_NOM_</nom>\n\t\t<prenom>_PRENOM_</prenom>\n\t\t<adresse>_ADRESSE_</adresse>\n\t\t<email>_EMAIL_</email>\n\t</personne>\n</annuaire>\n", axiome.xml());
		
		// Fermeture d'un premier incomplet
		
		setUpRacinePersonneVide();
		
		try {
			as.insererTerminal("nom", aucunAttribut);
			as.fermer("nom");
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("manquants"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Fermeture d'un premier complet
		
		setUpRacinePersonneVide();
		
		try {
			as.insererTerminal("nom", aucunAttribut);
			as.insererTerminalTexte("_NOM_");
			as.fermer("nom");

			assertTrue(true);
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Fermeture d'un suivant incomplet
		
		setUpRacinePersonneNomPrenom();
		
		try {
			as.insererTerminal("naissance", aucunAttribut);
			as.fermer("naissance");
			fail();
		} catch (ExceptionSyntaxique e) {
			assertTrue(e.getMessage().contains("manquants"));
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		// Fermeture d'un suivant complet
		
		setUpRacinePersonneNomPrenom();
		
		try {
			as.insererTerminal("adresse", aucunAttribut);
			as.insererTerminalTexte("_ADRESSE_");
			as.fermer("adresse");

			assertTrue(true);
			
			as.insererTerminal("email", aucunAttribut);
			as.insererTerminalTexte("_EMAIL_");
			as.fermer("email");			
		} catch (ExceptionSyntaxique e) {
			fail(e.getMessage());
		} catch (InstantiationException e) {
			fail(e.getMessage());
		}
		
		assertEquals("<annuaire>\n\t<personne>\n\t\t<nom>_NOM_</nom>\n\t\t<prenom>_PRENOM_</prenom>\n\t\t<adresse>_ADRESSE_</adresse>\n\t\t<email>_EMAIL_</email>\n\t</personne>\n</annuaire>\n", axiome.xml());
	}
}