package Tests;

import java.util.ArrayList;
import junit.framework.TestCase;
import Processeur.*;

public class InterpreteurXpathTest extends TestCase {
	protected UniteLexicale axiome;

	protected void setUp(String fichier) {
		axiome = Analyseur.analyser(fichier, "annuaire");
	}

	public void testInterpreter(){
		setUp("src/Tests/xpathTest1.xml");

		/** REQUETE COMMANCANT AVEC UN '/' **/

		/** Requete bien formee **/

		// Requete bien formee avec un seul element Annuaire
		String requete = "/annuaire";
		ArrayList<UniteLexicale> gauche,droit;

		try {
			InterpreteurXPath.interpreter(axiome,requete);

			gauche = InterpreteurXPath.interpreter(axiome, requete);
			droit = new ArrayList<UniteLexicale>();
			droit.add(axiome);

			for(int i =0 ; i<gauche.size(); i++)
				assertEquals(gauche.get(i).xml(),droit.get(i).xml());

		} catch (ExceptionXPath e) {
			fail();
		}	

		//Requete bien formee avec plus d'un niveau d'element
		requete = "/annuaire/personne/nom";

		try {
			InterpreteurXPath.interpreter(axiome,requete);
			gauche = InterpreteurXPath.interpreter(axiome, requete);
			droit = axiome.getFilsIndirectsParNom("nom");
			assertEquals(gauche.toString(),droit.toString());

			for(int i =0 ; i<gauche.size(); i++)
				assertEquals(gauche.get(i).xml(),droit.get(i).xml());

			//voir ce qui vaut le mieux, test avec les noeuds ou directement sur les chaines resultantes
			requete = "/annuaire/personne/nom";
			
			InterpreteurXPath.interpreter(axiome,requete);
			gauche = InterpreteurXPath.interpreter(axiome, requete);
			
			String droite= "<nom>Dupond</nom>\n";			
			assertEquals(gauche.get(0).xml(),droite);
			
			droite ="<nom>Ploum</nom>\n";
			assertEquals(gauche.get(1).xml(),droite);
			
			droite ="<nom>Robin</nom>\n";
			assertEquals(gauche.get(2).xml(),droite);

		} catch (ExceptionXPath e) {
			fail();
		}

		// Requete bien formee ne retournant aucun resultat

		setUp("src/Tests/xpathTest2.xml");
		requete = "/annuaire/personne/naissance";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());
			
		} catch (ExceptionXPath e) {
			fail();
		}

		/** Requete mal formee **/
		
		// Requete ne commencant pas par l'element racine
		requete = "/personne/nom";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());
			
		} catch (ExceptionXPath e) {
			fail();
		}

		// Requete contenant un element inconnu
		requete = "/annuaire/personne/fax";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());
			
		} catch (ExceptionXPath e) {
			fail();
		}

		// Requete ne respectant pas l'ordre des elements
		requete = "/annuaire/nom/personne";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());
			
		} catch (ExceptionXPath e) {
			fail();
		}

		// Requete se terminant mal
		requete ="/annuaire/";
		
		try {
			InterpreteurXPath.interpreter(axiome,requete);
			fail();
			
		} catch (ExceptionXPath e) {
			assertTrue(true);
		}

		// Requete vide
		requete ="/";
		
		try {
			InterpreteurXPath.interpreter(axiome,requete);
			fail();
			
		} catch (ExceptionXPath e) {
			assertTrue(true);
		}

		/** REQUETE COMMENCANT PAR '//'  (ou incluant des //) **/
		
		/** Requete bien formee **/

		// Requete indiquant directemnt le dernier niveau, commancant par '//'
		setUp("src/Tests/xpathTest1.xml");
		requete = "//email";
		
		try {
			gauche= InterpreteurXPath.interpreter(axiome,requete);
			String droite= "<email>julien@vaubourg.com</email>\n"; 
			assertEquals(gauche.get(0).xml(),droite);
			
			droite ="<email>tristan@vib.com</email>\n";
			assertEquals(gauche.get(1).xml(),droite);
			
			droite ="<email>julien@vib.com</email>\n";
			assertEquals(gauche.get(2).xml(),droite);

		} catch (ExceptionXPath e) {
			fail();
		}

		// '//' au sein de la requete
		requete ="/annuaire//naissance/annee";
		
		try {
			gauche= InterpreteurXPath.interpreter(axiome,requete);
			String droite ="<annee>1980</annee>\n";
			
			assertEquals(gauche.get(0).xml(),droite);
			droite = "<annee>1993</annee>\n";
			
			assertEquals(gauche.get(1).xml(),droite);
		} catch (ExceptionXPath e) {
			fail();
		}

		/** Requete mal formee */
		
		// Requete vide
		requete = "//";
		
		try {
			InterpreteurXPath.interpreter(axiome,requete);
			fail();
			
		} catch (ExceptionXPath e) {
			
			assertTrue(true);
		}

		/** REQUETE COMPRENANT DES PREDICATS **/

		/** Predicats bien formes **/

		// Predicat numerique indiquant le numero du noeud (element retourne = element sur lequel porte le predicat)
		setUp("src/Tests/xpathTest1.xml");
		requete = "//personne[2]";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			droit = axiome.getFilsIndirectsParNom("personne");
			UniteLexicale droite = droit.get(1);
			
			assertEquals(gauche.get(0).xml(),droite.xml());
		} catch (ExceptionXPath e) {
			fail();
		}

		// Predicat numerique indiquant le numero du noeud (element retourne != element sur lequel porte le predicat)
		requete = "//personne[2]/nom";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			droit = axiome.getFilsIndirectsParNom("nom");
			UniteLexicale droite = droit.get(1);
			
			assertEquals(gauche.get(0).xml(),droite.xml());
		} catch (ExceptionXPath e) {
			fail();
		}

		// Predicat testant l'existence du noeud
		requete = "//personne[naissance]";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			droit = axiome.getFilsIndirectsParNom("personne");
			
			int i = 1;
			
			for(UniteLexicale g : gauche)
				assertEquals(g.xml(), droit.get(i++).xml());

		} catch (ExceptionXPath e) {
			fail();
		}
		
		// Requete avec plusieurs predicats sur differents noeuds
		
		requete = "//personne[telephone]/telephone[1]";
		
		try {
			
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			droit = axiome.getFilsIndirectsParNom("telephone");
			
			int i = 0;
			
			for(UniteLexicale g : gauche)
				assertEquals(g.xml() ,droit.get(i++).xml());

		} catch (ExceptionXPath e) {
			fail();
		}
		
		/** Predicats mal formes **/

		// Predicat numerique hors de l'index
		requete = "//personne[16]";

		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());
			
		} catch (ExceptionXPath e) {
			fail();
		}

		// Caractere non attendus dans le predicat
		requete = "//personne[//]";
		
		try {
			InterpreteurXPath.interpreter(axiome,requete);
			fail();

		} catch (ExceptionXPath e) {
			assertTrue(true);
		}

		/** REQUETE COMPRENANT DES ATTRIBUTS **/

		/** Requete bien formee **/
		
		// Requete sur l'attribut
		requete = "//telephone/@type";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertEquals(gauche.get(0).xml() ,"gsm\n");
			assertEquals(gauche.get(1).xml() ,"fix\n");
			
		} catch (ExceptionXPath e) {
			fail();
		}

		// Requete avec predicat sur l'attribut
		requete = "//telephone[@type]";
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			droit = axiome.getFilsIndirectsParNom("telephone");
			
			int i = 0;
			
			for(UniteLexicale g : gauche)
				assertEquals(g.xml() ,droit.get(i++).xml());

		} catch (ExceptionXPath e) {
			fail();
		}
		
		//Requete avec predicat sur le contenu de l'attribut
		requete = "//telephone[@type=\"gsm\"]"; 
		
		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			droit = axiome.getFilsIndirectsParNom("telephone");
			assertEquals(gauche.get(0).xml(), droit.get(0).xml());

		} catch (ExceptionXPath e) {
			fail();
		}

		/** Requete mal formee ou vide**/

		// Requete avec un attribut non existant
		requete = "//telephone[@epyt]";

		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());

		} catch (ExceptionXPath e) {
			fail();
		}

		/** REQUETE AVEC LA FONCTION text() **/

		/** Requete bien formee **/
		
		// Affichage du texte d'un noeud (sans les balises)
		requete ="//nom[1]/text()";

		try {
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			String droite = "Dupond\n";

			assertEquals(gauche.get(0).xml(),droite);
		} catch (ExceptionXPath e) {
 			fail();
		}

		/** Requete mal formee ou vide**/

		// Appel de la fonction test sur un noeud n'en contenant pas
		requete = "//personne/text()";
		
		try{
			gauche = InterpreteurXPath.interpreter(axiome,requete);
			assertTrue(gauche.isEmpty());
			
		} catch (ExceptionXPath e) {
			fail();
		}
	}
}